<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\windows_network_drive\tests;

use OCP\Files\StorageNotAvailableException;

abstract class wndTest extends \PHPUnit_Framework_TestCase {

    protected $config;
    protected static $random;

    public static function setUpBeforeClass(){
        self::$random = self::getRandomCode(8);
    }

    public function test_getId() {
        $root = $this->wnd->getRoot();
        $share = $this->wnd->getShareName();
        if ( ! $root || $root[0]!='/') {
            $root='/'.$root;
        }
        if (substr($root, -1, 1)!='/') {
            $root.='/';
        }
        $share = trim($share, '/');
        $id = 'wnd::'.$this->wnd->getUser().'@'.$this->wnd->getHost().'/'.$share.'/'.$root;
        $this->assertEquals($this->wnd->getId(), $id);
    }

    public function test_mkdir() {
        if ($this->wnd->isCreatable("")) {
            $this->assertTrue($this->wnd->mkdir('unittest'.self::$random));
            $this->assertTrue($this->wnd->isUpdatable('unittest'.self::$random));
            $this->assertTrue($this->wnd->isCreatable('unittest'.self::$random));
            $this->assertTrue($this->wnd->isReadable('unittest'.self::$random));
            $this->assertTrue($this->wnd->mkdir('unittest'.self::$random.'/2'));
            $this->assertTrue($this->wnd->mkdir('unittest'.self::$random.'/3'));
        } else {
            $this->markTestSkipped('cannot create a dir inside a non-updatable folder');
        }
    }

    public function test_mkdir_same_dir() {
        if ($this->wnd->isCreatable("")) {
            $this->assertFalse($this->wnd->mkdir('unittest'.self::$random));
        } else {
            $this->markTestSkipped('cannot create a dir inside a non-updatable folder');
        }
    }

    public function test_mkdir_bad_dir() {
        if ($this->wnd->isCreatable("")) {
            $this->assertFalse($this->wnd->mkdir('unittest'.self::$random.'/aaaa/bbbb'));
        } else {
            $this->markTestSkipped('cannot create a dir inside a non-updatable folder');
        }
    }

    public function test_hasUpdated() {
        $this->assertTrue($this->wnd->hasUpdated("", 0));
        $mtime = $this->wnd->filemtime("");
        $this->assertFalse($this->wnd->hasUpdated("", $mtime + 90));
        if ($this->wnd->isCreatable("")) {
            $this->assertTrue($this->wnd->hasUpdated('unittest'.self::$random, 0));
        }
    }

    public function test_hasUpdated_wrong() {
        // if something is wrong, hasUpdated return true to force an update
        $this->setExpectedException('\OCP\Files\StorageNotAvailableException');
        $this->wndwrong->hasUpdated('', 0);
        $this->setExpectedException('\OCP\Files\StorageNotAvailableException');
        $this->wndwrong->hasUpdated('unittest'.self::$random, 0);
        $this->setExpectedException('\OCP\Files\StorageNotAvailableException');
        $this->wndwrong->hasUpdated(self::$random, 0);
    }

    public function test_stat() {
        $response = $this->wnd->stat("");
        $this->assertArrayHasKey('size', $response);
        $this->assertArrayHasKey('mtime', $response);
    }

    public function test_stat_bad_path() {
        //$this->setExpectedException('PHPUnit_Framework_Error');
        //$this->wnd->stat('unittest'.self::$random.'/non.txt');
        $this->assertFalse($this->wnd->stat('unittest'.self::$random.'/non.txt'));
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_stat_wrong_1() {
        $this->wndwrong->stat('');
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_stat_wrong_2() {
        $this->wndwrong->stat('unittest' . self::$random);
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_stat_wrong_3() {
        $this->wndwrong->stat('unittest'.self::$random.'/non.txt');
    }

    public function test_filetype_dir() {
        $this->assertEquals('dir', $this->wnd->filetype(""));
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_filetype_wrong1() {
        $this->wndwrong->filetype("");
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_filetype_wrong2() {
        $this->wndwrong->filetype('unittest'.self::$random);
    }

    public function test_isReadable() {
        $this->assertTrue($this->wnd->isReadable(""));
        $this->assertFalse($this->wnd->isReadable('unittest_fake'));
        $this->assertFalse($this->wnd->isReadable('unittest_fake/ffee/ooo'));
    }

    public function test_isUpdatable() {
        // missing files and folder neither
        $this->assertFalse($this->wnd->isUpdatable('unittest_fake'));
        $this->assertFalse($this->wnd->isUpdatable('unittest_fake/ffee/ooo'));
    }

    public function test_isSharable() {
        // share disabled
        $this->assertFalse($this->wnd->isSharable(''));
        $this->assertFalse($this->wnd->isSharable('unittest'.self::$random));
        $this->assertFalse($this->wnd->isSharable('unittest_fake'));
        $this->assertFalse($this->wnd->isUpdatable('unittest_fake/ffee/ooo'));
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_permissions_wrong_readable1() {
        $this->wndwrong->isReadable('');
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_permissions_wrong_readable2() {
        $this->wndwrong->isReadable('unittest' . self::$random);
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_permissions_wrong_readable3() {
        $this->wndwrong->isReadable('unittest_fake');
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_permissions_wrong_updateable1() {
        $this->assertFalse($this->wndwrong->isUpdatable(''));
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_permissions_wrong_updateable2() {
        $this->assertFalse($this->wndwrong->isUpdatable('unittest' . self::$random));
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_permissions_wrong_updateable3() {
        $this->assertFalse($this->wndwrong->isUpdatable('unittest_fake'));
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_permissions_wrong_creatable1() {
        $this->assertFalse($this->wndwrong->isCreatable(''));  // updateable false
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_permissions_wrong_creatable2() {
        $this->assertFalse($this->wndwrong->isCreatable('unittest' . self::$random));
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_permissions_wrong_creatable3() {
        $this->assertFalse($this->wndwrong->isCreatable('unittest_fake'));
    }

    public function test_getMimeType_dir() {
        $this->assertEquals('httpd/unix-directory', $this->wnd->getMimeType(""));
    }

    public function test_fopen_write() {
        if ($this->wnd->isCreatable("")) {
            // the unittest.(random) should have been created at this point
            $tmpfile = $this->wnd->fopen('unittest'.self::$random.'/test_file.txt' , 'wb');
            $this->assertEquals('resource', gettype($tmpfile));
            fwrite($tmpfile, 'writing to file');
            fclose($tmpfile);
            $this->assertTrue($this->wnd->file_exists('unittest'.self::$random.'/test_file.txt'));
        } else {
            $this->markTestSkipped('we cannot create the test folder so we cannot create a file there');
        }
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_fopen_write_wrong() {
        $this->wndwrong->fopen('unittest'.self::$random.'/test_file2.txt' , 'wb');
    }

    public function test_fopen_read() {
        if ($this->wnd->isCreatable("")) {
            $tmpfile = $this->wnd->fopen('unittest'.self::$random.'/test_file.txt' , 'rb');
            $this->assertEquals('resource', gettype($tmpfile));
            $content = fgets($tmpfile, 4096);
            fclose($tmpfile);
            $this->assertEquals('writing to file', $content);
        } else {
            $this->markTestSkipped('we cannot create the test folder so we cannot get a file there');
        }
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_fopen_read_wrong() {
        $this->wndwrong->fopen('unittest'.self::$random.'/test_file.txt' , 'rb');
    }

    public function test_filetype_file() {
        if ($this->wnd->isCreatable("")) {
            $this->assertEquals('file', $this->wnd->filetype('unittest'.self::$random.'/test_file.txt'));
        } else {
            $this->markTestSkipped('we cannot create the test folder so we cannot create a file and get the filetype');
        }
    }

    public function test_file_exists() {
        $this->assertTrue($this->wnd->file_exists(''));  // root always exists
        $this->assertFalse($this->wnd->file_exists(self::$random));  // missing file
        if ($this->wnd->isCreatable("")) {
            $this->assertTrue($this->wnd->file_exists('unittest'.self::$random.'/test_file.txt'));
        } else {
            $this->markTestSkipped('we cannot create the test folder so check if a file exists there');
        }
    }

    public function test_file_no_exists() {
        $this->assertFalse($this->wnd->file_exists('unittest'.self::$random.'/test_no_file.txt'));
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_file_exists_wrong() {
        $this->wndwrong->file_exists('unittest'.self::$random.'/test_file.txt');
    }

    public function test_getMimeType_file() {
        if ($this->wnd->isCreatable("")) {
            $this->assertEquals('text/plain', $this->wnd->getMimeType('unittest'.self::$random.'/test_file.txt'));
        } else {
            $this->markTestSkipped('we cannot create the test folder so we cannot get the mimetype of a file there');
        }
    }

    public function test_getMimeType_file_missing() {
        $this->assertFalse($this->wnd->getMimeType('lkjdsfkjh.txt'));
    }

    public function test_copy_file() {
        if ($this->wnd->isCreatable("")) {
            $this->assertTrue($this->wnd->copy('unittest'.self::$random.'/test_file.txt', 'unittest'.self::$random.'/copied.txt'));
        } else {
            $this->markTestSkipped('our root folder is not writeable');
        }
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_copy_file_wrong1() {
        $this->wndwrong->copy('unittest' . self::$random . '/test_file.txt', 'unittest' . self::$random . '/copied2.txt');
    }
    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_copy_file_wrong2() {
        // wrong wnd won't find the "copied" file
        try {
            $this->wndwrong->copy('unittest' . self::$random . '/test_file.txt', 'unittest' . self::$random . '/copied2.txt');
        } catch (StorageNotAvailableException $e) {}
        $d = $this->wndwrong->file_exists('unittest' . self::$random . '/copied2.txt');
    }
    public function test_copy_file_wrong3() {
        try {
            $this->wndwrong->copy('unittest' . self::$random . '/test_file.txt', 'unittest' . self::$random . '/copied2.txt');
        } catch (StorageNotAvailableException $e) {}
        // right wnd won't find the "copied" file because it hasn't been copied
        $this->assertFalse($this->wnd->file_exists('unittest'.self::$random.'/copied2.txt'));
    }

    public function test_rename_dir() {
        if ($this->wnd->isCreatable("")) {
            $this->assertTrue($this->wnd->rename('unittest'.self::$random, 'unittest_renamed'));
            $this->assertTrue($this->wnd->rename('unittest_renamed', 'unittest'.self::$random));
        } else {
            $this->markTestSkipped('our root folder is not writeable');
        }
    }
    public function test_rename_file() {
        if ($this->wnd->isCreatable("")) {
            $this->assertTrue($this->wnd->rename('unittest'.self::$random.'/test_file.txt', 'unittest'.self::$random.'/renamed.txt'));
            $this->assertTrue($this->wnd->rename('unittest'.self::$random.'/renamed.txt', 'unittest'.self::$random.'/test_file.txt'));
            $this->assertTrue($this->wnd->rename('unittest'.self::$random.'/copied.txt', 'unittest'.self::$random.'/test_file.txt'));
            $this->assertFalse($this->wnd->rename('unittest'.self::$random.'/98712', 'unittest'.self::$random.'/test_file.txt'));
        } else {
            $this->markTestSkipped('our root folder is not writeable');
        }
    }

    public function test_touch() {
        if ($this->wnd->isCreatable("")) {
            $this->assertTrue($this->wnd->touch('unittest'.self::$random.'/test_touch_file.txt'));
        } else {
            $this->markTestSkipped('our root folder is not writeable');
        }
    }

    public function test_touch_fail() {
        $this->assertFalse($this->wnd->touch('unittest'.self::$random.'/test_touch_file.txt', 198332));
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_touch_wrong() {
        $this->assertFalse($this->wndwrong->touch('unittest'.self::$random.'/test_touch_file2.txt'));
    }

    public function test_opendir() {
        $this->assertEquals('resource', gettype($this->wnd->opendir("")));
        // check for missing directory
        $dir = $this->wnd->opendir(self::$random);  // this shouldn't exists
        $this->assertEquals('resource', gettype($dir));
        $this->assertFalse(readdir($dir));  // it must return empty contents
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_opendir_wrong() {
        $this->wndwrong->opendir("");
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_unlink_wrong() {
        $this->wndwrong->unlink('unittest'.self::$random);
    }

    public function test_unlink() {
        if ($this->wnd->isCreatable("")) {
            $this->assertTrue($this->wnd->unlink('unittest'.self::$random.'/test_touch_file.txt'));
            // unlink of dirs will fallback to rmdir
            $this->assertTrue($this->wnd->unlink('unittest'.self::$random.'/3'));
        } else {
            $this->markTestSkipped('our root folder is not writeable');
        }
    }

    /**
     * @expectedException  \OCP\Files\StorageNotAvailableException
     */
    public function test_rmdir_wrong() {
        $this->wndwrong->rmdir('unittest'.self::$random);
    }

    public function test_rmdir() {
        if ($this->wnd->isCreatable("")) {
            $this->assertTrue($this->wnd->rmdir('unittest'.self::$random));
        } else {
            $this->markTestSkipped('our root folder is not writeable');
        }
    }

    public function test_rmdir_non_exists_dir() {
        $this->assertFalse($this->wnd->rmdir('unittest'.self::$random));
    }

    public function test_password_reset() {
        try {
            # first call should throw an exception
            $this->wndbadpass->file_exists(self::$random);
        } catch (\OCP\Files\StorageNotAvailableException $ex) {
            $this->assertEmpty($this->wndbadpass->getPassword());
            try {
                $this->wndbadpass->file_exists(self::$random);
            } catch (\OCP\Files\StorageNotAvailableException $ex) {
                # we should be checking for something other than the message, but nothing useful
                # at this point to distinguish between the first and second exceptions
                $this->assertEquals($ex->getMessage(), 'Password required when username given');
                return;
            }
        }
        $this->fail('StorageNotAvailableException not thrown');
    }

    public static function getRandomCode($size){
        $an = 'abcdefghijklmnopqrstuvwxyz0123456789';
        $su = strlen($an) - 1;
        $result = "";
        for($i=0; $i<$size; $i++){
            $result .= substr($an, rand(0, $su), 1);
        }
        return $result;
    }

}
